# $Id: /mirror/datetime/DateTime-Util-Astro/trunk/lib/DateTime/Util/Astro.pm 31668 2007-12-04T03:34:52.383554Z lestrrat  $
#
# Copyright (c) 2004-2007 Daisuke Maki <daisuke@endeworks.jp>

package DateTime::Util::Astro;
use strict;
our $VERSION = '0.12000';

1;

__END__

=head1 NAME

DateTime::Util::Astro - Utilities for Astronomical Calculations

=head1 DESCRIPTION

See DateTime::Util::Astro::Moon and DateTime::Util::Astro::Sun

=head1 LICENSE

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

See http://www.perl.com/perl/misc/Artistic.html

=head1 AUTHOR

Daisuke Maki E<lt>daisuke@endeworks.jpE<gt>

=cut
